// Copyright (C) 2012-2014 ChaosForge Ltd
// http://chaosforge.org/
//
// This file is part of NV Libraries.
// For conditions of distribution and use, see copyright notice in nv.hh

/**
 * @file gui_environment.hh
 * @author Kornel Kisielewicz
 * @brief GUI Environment (root)
 */

#ifndef NV_GUI_ENVIRONMENT_HH
#define NV_GUI_ENVIRONMENT_HH

#include <nv/gui/gui_element.hh>
#include <nv/gui/gui_style.hh>
#include <nv/lua/lua_state.hh>
#include <nv/core/io_event.hh>
#include <nv/interface/window.hh>
#include <nv/core/array.hh>

namespace nv
{

	namespace gui
	{

		class environment
		{
		public:
			environment( window* w );
			// temporary
			void load_style( const std::string& filename );
			handle create_element( const rectangle& r );
			handle create_element( handle parent, const rectangle& r );
			void set_text( handle e, const string& text );
			void set_class( handle  e, const string& text );
			void update();
			void draw();
			void destroy_element( handle e );
			bool process_io_event( const io_event& ev );
			virtual ~environment();
		protected:
			handle get_element( const position& p );
			void add_child( handle child );
			void add_child( handle parent, handle child );
			void remove_child( handle parent, handle child );
			void destroy_children( handle e );
			void update( handle e, uint32 elapsed );
			void draw( handle e );
			bool process_io_event( handle e, const io_event& ev );
			handle get_deepest_child( handle e, const position& p );
			void move_to_top( handle child );
			void move_to_bottom( handle child );
			void set_relative( handle e, const rectangle& r );
			void set_relative( handle e, const position& p );
			void recalculate_absolute( handle e );
			
			entity_store< element, handle > m_elements;
			renderer*     m_renderer;
			window*       m_window;
			handle        m_screen;
			rectangle     m_area;
		};

	} // namespace gui

} // namespace nv

#endif // NV_GUI_ENVIRONMENT_HH
